"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IconaBridgeClient = exports.ICONA_BRIDGE_PORT = void 0;
const net_1 = __importDefault(require("net"));
const types_1 = require("./types");
const promise_socket_1 = __importDefault(require("promise-socket"));
const packet_1 = require("./icona/packet");
const chalk_1 = __importDefault(require("chalk"));
const utils_1 = require("./utils");
exports.ICONA_BRIDGE_PORT = 64100;
function accessMessage(requestId, token) {
    const json = {
        message: 'access',
        'user-token': token,
        'message-type': 'request',
        'message-id': types_1.ViperChannelType.UAUT,
    };
    return packet_1.PacketMessage.createJSONPacket(requestId, json);
}
function getConfigMessage(requestId, addressbooks) {
    const json = {
        message: 'get-configuration',
        addressbooks: addressbooks,
        'message-type': 'request',
        'message-id': types_1.ViperChannelType.UCFG,
    };
    return packet_1.PacketMessage.createJSONPacket(requestId, json);
}
function getPushInfoMessage(requestId, vip, deviceToken) {
    const json = {
        'apt-address': vip['apt-address'],
        'apt-subaddress': vip["apt-subaddress"],
        'bundle-id': 'com.comelitgroup.friendhome',
        message: 'push-info',
        'message-id': types_1.ViperChannelType.PUSH,
        'os-type': 'ios',
        'profile-id': '3',
        'device-token': deviceToken,
        'message-type': 'request',
    };
    return packet_1.PacketMessage.createJSONPacket(requestId, json);
}
// eslint-disable-next-line @typescript-eslint/no-unused-vars
function getInfoMessage(requestId) {
    const json = {
        message: 'server-info',
        'message-type': 'request',
        'message-id': types_1.ViperChannelType.SERVER_INFO,
    };
    return packet_1.PacketMessage.createJSONPacket(requestId, json);
}
function getUnknownOpenDoorMessage(requestId, vip) {
    const buffers = [
        Buffer.from([0xc0, 0x18, 0x5c, 0x8b]),
        Buffer.from([0x2b, 0x73, 0x00, 0x11]),
        Buffer.from([0x00, 0x40, 0xac, 0x23]),
        (0, utils_1.stringToBuffer)(`${vip["apt-address"]}${vip["apt-subaddress"]}`, true),
        Buffer.from([0x10, 0x0e]),
        Buffer.from([0x00, 0x00, 0x00, 0x00]),
        Buffer.from([0xff, 0xff, 0xff, 0xff]),
        (0, utils_1.stringToBuffer)(`${vip["apt-address"]}${vip["apt-subaddress"]}`, true),
        (0, utils_1.stringToBuffer)(`${vip["apt-address"]}`, true),
        utils_1.NULL
    ];
    return packet_1.PacketMessage.createBinaryPacketFromBuffers(requestId, ...buffers);
}
function getOpenDoorMessage(requestId, vip, doorItem, confirm = false) {
    const buffers = [
        confirm ? Buffer.from([packet_1.MessageType.OPEN_DOOR_CONFIRM]) : Buffer.from([packet_1.MessageType.OPEN_DOOR]),
        Buffer.from([0x5c, 0x8b]),
        Buffer.from([0x2c, 0x74, 0x00, 0x00]),
        Buffer.from([0xff, 0xff, 0xff, 0xff]),
        (0, utils_1.stringToBuffer)(`${vip["apt-address"]}${vip["apt-subaddress"]}`, true),
        doorItem ? (0, utils_1.stringToBuffer)(`${doorItem["apt-address"]}`) : (0, utils_1.stringToBuffer)(`${vip["apt-address"]}`, true),
        utils_1.NULL
    ];
    return packet_1.PacketMessage.createBinaryPacketFromBuffers(requestId, ...buffers);
}
function getInitOpenDoorMessage(requestId, vip, doorItem) {
    const buffers = [
        Buffer.from([0xc0, 0x18, 0x70, 0xab]),
        Buffer.from([0x29, 0x9f, 0x00, 0xd]),
        Buffer.from([0x00, 0x2d]),
        (0, utils_1.stringToBuffer)(`${doorItem["apt-address"]}`, true),
        utils_1.NULL,
        Buffer.from([doorItem["output-index"], 0x0, 0x0, 0x0]),
        Buffer.from([0xff, 0xff, 0xff, 0xff]),
        (0, utils_1.stringToBuffer)(`${vip["apt-address"]}${vip["apt-subaddress"]}`, true),
        (0, utils_1.stringToBuffer)(`${doorItem["apt-address"]}`, true),
        utils_1.NULL
    ];
    return packet_1.PacketMessage.createBinaryPacketFromBuffers(requestId, ...buffers);
}
class IconaBridgeClient {
    constructor(host, port = exports.ICONA_BRIDGE_PORT, logger = console) {
        this.openChannels = new Map();
        this.host = host;
        this.port = port;
        this.logger = logger;
        this.id = Math.round(Math.random() * 10 + 8000);
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            this._socket = new net_1.default.Socket();
            this.socket = new promise_socket_1.default(this._socket);
            this.socket.setTimeout(5000);
            this.logger.info(`Connecting to ${this.host}:${this.port}`);
            yield this.socket.connect(this.port, this.host);
            this.socket.socket.setMaxListeners(40);
            this.logger.info('connected');
        });
    }
    writeBytePacket(packet) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug(`Writing bytes to socket: \n${(0, utils_1.bytesToHex)(packet.bytes)}`);
            yield this.socket.writeAll(packet.bytes);
        });
    }
    readResponse() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const header = (yield this.socket.read(8));
                const size = header.readUIntLE(2, 2);
                let requestId = header.readUIntLE(4, 4);
                const body = (yield this.socket.read(size));
                this.logger.debug(`Read bytes from socket (size: ${size}):\n${(0, utils_1.bytesToHex)(body)}`);
                return this.decodeResponse(requestId, body);
            }
            catch (e) {
                this.logger.warn('No bytes to read, skipping');
                return null;
            }
        });
    }
    decodeResponse(requestId, body) {
        if (requestId === 0) {
            // Request ID is at the end of the message for binary protocol
            const number = body.readUIntLE(0, 2);
            const seq = body.readUIntLE(2, 2);
            switch (number) {
                case packet_1.MessageType.COMMAND: {
                    const subSize = body.readUIntLE(4, 4);
                    requestId = body.readUIntLE(subSize, subSize);
                    return { requestId, sequence: seq, type: packet_1.BinaryResponseType.BINARY };
                }
                case packet_1.MessageType.END:
                    this.logger.info("END response received with sequence number " + seq);
                    return { requestId, sequence: seq, type: packet_1.BinaryResponseType.BINARY };
                default:
                    this.logger.info("No handler implemented for message type " + (0, packet_1.getInt64Bytes)(number).map(n => `0x${n}`).join(' '));
                    return null;
            }
        }
        else { // data packet
            const first = body.readUIntLE(0, 1);
            const second = body.readUIntLE(1, 1);
            switch (first) {
                case 0x7b:
                    // the first char is an open curly bracket (JSON MESSAGE)
                    return { requestId, sequence: packet_1.NO_SEQ, type: packet_1.BinaryResponseType.JSON, json: (0, packet_1.readJSON)(body) };
                default:
                    // binary data (still not fully reversed)
                    return this.decodeBinaryResponse(requestId, first, second, body);
            }
        }
    }
    decodeBinaryResponse(requestId, subtype, type, body) {
        switch (type) {
            case 0x18: // open door first response
                this.logger.debug('Open door: ' + (0, packet_1.bufferToASCIIString)(body));
        }
        return { requestId, sequence: packet_1.NO_SEQ, type: packet_1.BinaryResponseType.BINARY };
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.socket.end();
        });
    }
    openChanel(channel, additionalData) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.openChannels.has(channel)) {
                return Promise.resolve(this.openChannels.get(channel));
            }
            this.id++;
            const openChannelData = { channel, sequence: 1, id: this.id };
            this.openChannels.set(channel, openChannelData);
            const p = packet_1.PacketMessage.createBinaryPacketFromStrings(this.id, 1, packet_1.MessageType.COMMAND, channel, additionalData);
            yield this.writeBytePacket(p);
            const response = yield this.readResponse();
            if (response) {
                this.logger.debug(`Opened channel ${channel}, sequence is ${response.sequence} and requestId is ${(0, utils_1.numberToHex)(this.id)}`);
                if (response.type === packet_1.BinaryResponseType.BINARY && response.sequence === 2) {
                    openChannelData.sequence = response.sequence;
                    return openChannelData;
                }
            }
            this.logger.error(chalk_1.default.red(`Error trying opening channel ${channel}, received wrong response from server`));
            yield this.shutdown();
        });
    }
    closeChanel(channelData) {
        return __awaiter(this, void 0, void 0, function* () {
            const p = packet_1.PacketMessage.createBinaryPacketFromStrings(channelData.id, ++channelData.sequence, packet_1.MessageType.END);
            yield this.writeBytePacket(p);
            const response = yield this.readResponse();
            this.logger.debug(`Closed channel ${channelData.channel}, sequence id is ${response.sequence} and requestId is ${(0, utils_1.numberToHex)(channelData.id)}`);
            if (response.type === packet_1.BinaryResponseType.BINARY && response.sequence === channelData.sequence + 1) {
                this.openChannels.delete(channelData.channel);
                return Object.assign(Object.assign({}, channelData), { sequence: response.sequence });
            }
            this.logger.error(chalk_1.default.red(`Error trying closing channel ${channelData.channel}, received wrong response from server`));
            yield this.shutdown();
        });
    }
    authenticate(token) {
        return __awaiter(this, void 0, void 0, function* () {
            const channelData = yield this.openChanel(packet_1.Channel.UAUT);
            const message = accessMessage(channelData.id, token);
            yield this.writeBytePacket(message);
            const resp = yield this.readResponse();
            if (resp && resp.json) {
                yield this.closeChanel(channelData);
                return resp.json["response-code"];
            }
            this.logger.error(chalk_1.default.red(`Error trying to authenticate, received wrong response from server: ${JSON.stringify(resp)}`));
            yield this.shutdown();
            return 500;
        });
    }
    getConfig(addressbooks, closeChannel = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const channelData = yield this.openChanel(packet_1.Channel.UCFG);
            if (channelData) {
                const packetMessage = getConfigMessage(channelData.id, addressbooks);
                yield this.writeBytePacket(packetMessage);
                const resp = yield this.readResponse();
                if (resp && resp.type === packet_1.BinaryResponseType.JSON) {
                    if (closeChannel) {
                        yield this.closeChanel(channelData);
                    }
                    return resp.json;
                }
                this.logger.error(chalk_1.default.red(`Error trying to get configuration, received wrong response from server: ${JSON.stringify(resp)}`));
            }
            return null;
        });
    }
    getServerInfo(closeChannel = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const channelData = yield this.openChanel(packet_1.Channel.INFO);
            if (channelData) {
                const packetMessage = getInfoMessage(channelData.id);
                yield this.writeBytePacket(packetMessage);
                const resp = yield this.readResponse();
                if (resp && resp.type === packet_1.BinaryResponseType.JSON) {
                    if (closeChannel) {
                        yield this.closeChanel(channelData);
                    }
                    return resp.json;
                }
                this.logger.error(chalk_1.default.red(`Error trying to get server info, received wrong response from server: ${JSON.stringify(resp)}`));
            }
            return null;
        });
    }
    getPushInfo(vip, deviceToken, closeChannel = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const channelData = yield this.openChanel(packet_1.Channel.PUSH);
            if (channelData) {
                const packetMessage = getPushInfoMessage(channelData.id, vip, deviceToken);
                yield this.writeBytePacket(packetMessage);
                const resp = yield this.readResponse();
                if (resp && resp.type === packet_1.BinaryResponseType.JSON) {
                    if (closeChannel) {
                        yield this.closeChanel(channelData);
                    }
                    return resp.json;
                }
                this.logger.error(chalk_1.default.red(`Error trying to get push info, received wrong response from server: ${JSON.stringify(resp)}`));
            }
            return null;
        });
    }
    openDoorInit(vip) {
        return __awaiter(this, void 0, void 0, function* () {
            const ctpp = yield this.openChanel(packet_1.Channel.CTPP, `${vip["apt-address"]}${vip["apt-subaddress"]}`);
            const initMessage = getUnknownOpenDoorMessage(ctpp.id, vip);
            yield this.writeBytePacket(initMessage);
            const resp1 = yield this.readResponse();
            this.logger.debug(`CTPP 1:\n${JSON.stringify(resp1)}`);
            const resp2 = yield this.readResponse();
            this.logger.debug(`CTPP 2:\n${JSON.stringify(resp2)}`);
        });
    }
    openDoor(vip, doorItem) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.openChannels.has(packet_1.Channel.CTPP)) {
                yield this.openDoorInit(vip);
            }
            const channelCTPPData = this.openChannels.get(packet_1.Channel.CTPP);
            const packetMessage = getOpenDoorMessage(channelCTPPData.id, vip, null);
            yield this.writeBytePacket(packetMessage);
            const confirmMessage = getOpenDoorMessage(channelCTPPData.id, vip, null, true);
            yield this.writeBytePacket(confirmMessage);
            const message1 = getInitOpenDoorMessage(channelCTPPData.id, vip, doorItem);
            yield this.writeBytePacket(message1);
            const resp = yield this.readResponse();
            this.logger.debug(`${JSON.stringify(resp)}`);
            const resp2 = yield this.readResponse();
            this.logger.debug(`${JSON.stringify(resp2)}`);
            const packetMessage1 = getOpenDoorMessage(channelCTPPData.id, vip, doorItem);
            yield this.writeBytePacket(packetMessage1);
            const confirmMessage1 = getOpenDoorMessage(channelCTPPData.id, vip, doorItem, true);
            yield this.writeBytePacket(confirmMessage1);
        });
    }
}
exports.IconaBridgeClient = IconaBridgeClient;
//# sourceMappingURL=icona-bridge-client.js.map