import { ClimaMode, ThermoSeason } from './comelit-client';
export declare type DEVICE_STATUS = '0' | '1' | '2';
export interface DomoticData {
    id: string;
    type: OBJECT_TYPE;
    sub_type: OBJECT_SUBTYPE;
    sched_status?: DEVICE_STATUS;
    sched_lock?: string;
    schedZoneStatus?: number[];
    status: DEVICE_STATUS;
}
export interface DeviceData<T extends DeviceData = any> extends DomoticData {
    descrizione: string;
    placeOrder?: string;
    num_modulo?: string;
    num_uscita?: string;
    icon_id?: string;
    isProtected?: DEVICE_STATUS;
    objectId?: string;
    placeId?: string;
    powerst: DEVICE_STATUS;
    elements?: T[];
}
export interface DeviceInfo {
    id: string;
    data: DeviceData;
}
export interface OtherDeviceData extends DeviceData {
    tempo_uscita: string;
}
export interface LightDeviceData extends DeviceData {
}
export interface BlindDeviceData extends DeviceData {
    open_status?: DEVICE_STATUS;
    position?: string;
    openTime?: string;
    closeTime?: string;
    preferPosition?: string;
    enablePreferPosition?: DEVICE_STATUS;
}
export interface OutletDeviceData extends DeviceData {
    instant_power: string;
    out_power: number;
}
export interface IrrigationDeviceData extends DeviceData {
}
export interface ThermostatDeviceData extends DeviceData {
    num_ingresso?: number;
    num_moduloIE?: string;
    num_uscitaIE?: string;
    num_moduloI?: string;
    num_uscitaI?: string;
    num_moduloE?: string;
    num_uscitaE?: string;
    num_moduloI_ana?: string;
    num_uscitaI_ana?: string;
    num_moduloE_ana?: string;
    num_uscitaE_ana?: string;
    num_moduloUD?: string;
    num_uscitaUD?: string;
    num_moduloU?: string;
    num_uscitaU?: string;
    num_moduloD?: string;
    num_uscitaD?: string;
    num_moduloU_ana?: string;
    num_uscitaU_ana?: string;
    num_moduloD_ana?: string;
    num_uscitaD_ana?: string;
    night_mode?: string;
    soglia_man_inv?: string;
    soglia_man_est?: string;
    soglia_man_notte_inv?: string;
    soglia_man_notte_est?: string;
    soglia_semiauto?: string;
    soglia_auto_inv?: string;
    soglia_auto_est?: string;
    out_enable_inv?: DEVICE_STATUS;
    out_enable_est?: DEVICE_STATUS;
    dir_enable_inv?: DEVICE_STATUS;
    dir_enable_est?: DEVICE_STATUS;
    heatAutoFanDisable?: DEVICE_STATUS;
    coolAutoFanDisable?: DEVICE_STATUS;
    heatSwingDisable?: DEVICE_STATUS;
    coolSwingDisable?: DEVICE_STATUS;
    out_type_inv?: string;
    out_type_est?: string;
    temp_base_inv?: string;
    temp_base_est?: string;
    out_enable_umi?: string;
    out_enable_deumi?: string;
    dir_enable_umi?: string;
    dir_enable_deumi?: string;
    humAutoFanDisable?: string;
    dehumAutoFanDisable?: string;
    humSwingDisable?: string;
    dehumSwingDisable?: string;
    out_type_umi?: string;
    out_type_deumi?: string;
    soglia_man_umi?: string;
    soglia_man_deumi?: string;
    soglia_man_notte_umi?: string;
    soglia_man_notte_deumi?: string;
    night_mode_umi?: string;
    soglia_semiauto_umi?: string;
    umi_base_umi?: string;
    umi_base_deumi?: string;
    coolLimitMax?: string;
    coolLimitMin?: string;
    heatLimitMax?: string;
    heatLimitMin?: string;
    viewOnly?: string;
    temperatura?: string;
    auto_man?: ClimaMode;
    est_inv?: ThermoSeason;
    soglia_attiva?: string;
    out_value_inv?: string;
    out_value_est?: string;
    dir_out_inv?: string;
    dir_out_est?: string;
    semiauto_enabled?: string;
    umidita?: string;
    auto_man_umi?: ClimaMode;
    deumi_umi?: string;
    soglia_attiva_umi?: string;
    semiauto_umi_enabled?: string;
}
export interface SupplierDeviceData extends DeviceData {
    label_value: string;
    label_price: string;
    prod: string;
    count_div: string;
    cost: string;
    kCO2: string;
    compare: string;
    groupOrder: string;
    instant_power: string;
}
export declare type DeviceIndex<T = DeviceData> = Map<string, T>;
export declare enum OBJECT_TYPE {
    OTHER = 1,
    BLIND = 2,
    LIGHT = 3,
    IRRIGATION = 4,
    THERMOSTAT = 9,
    OUTLET = 10,
    POWER_SUPPLIER = 11,
    ZONE = 1001
}
export declare enum OBJECT_SUBTYPE {
    GENERIC = 0,
    DIGITAL_LIGHT = 1,
    RGB_LIGHT = 2,
    TEMPORIZED_LIGHT = 3,
    DIMMER_LIGHT = 4,
    OTHER_DIGIT = 5,
    OTHER_TMP = 6,
    ELECTRIC_BLIND = 7,
    CLIMA_TERM = 12,
    GENERIC_ZONE = 13,
    CONSUMPTION = 15,
    CLIMA_THERMOSTAT_DEHUMIDIFIER = 16,
    CLIMA_DEHUMIDIFIER = 17,
    ENHANCED_ELECTRIC_BLIND = 31
}
export declare const ON = 1;
export declare const OFF = 0;
export declare const OPEN = 1;
export declare const CLOSE = 0;
export declare const IDLE = 2;
export declare const STATUS_ON = "1";
export declare const STATUS_OFF = "0";
export declare const STATUS_OPEN = "1";
export declare const STATUS_CLOSED = "0";
export declare class HomeIndex {
    readonly othersIndex: DeviceIndex<OtherDeviceData>;
    readonly lightsIndex: DeviceIndex<LightDeviceData>;
    readonly roomsIndex: DeviceIndex;
    readonly thermostatsIndex: DeviceIndex<ThermostatDeviceData>;
    readonly blindsIndex: DeviceIndex<BlindDeviceData>;
    readonly outletsIndex: DeviceIndex<OutletDeviceData>;
    readonly irrigationIndex: DeviceIndex<IrrigationDeviceData>;
    readonly supplierIndex: DeviceIndex<SupplierDeviceData>;
    readonly unknownIndex: DeviceIndex;
    readonly mainIndex: DeviceIndex;
    constructor(home: DeviceData, logger?: ConsoleLike);
    get(id: string): DeviceData;
    updateObject(id: string, data: Readonly<DeviceData>): DeviceData;
    private visitElement;
}
export interface ConsoleLike {
    log: (message?: any, ...optionalParams: any[]) => void;
    info: (message?: any, ...optionalParams: any[]) => void;
    warn: (message?: any, ...optionalParams: any[]) => void;
    debug: (message?: any, ...optionalParams: any[]) => void;
    error: (message?: any, ...optionalParams: any[]) => void;
}
export declare const ViperChannelType: {
    SERVER_INFO: number;
    PUSH: number;
    UAUT: number;
    UCFG: number;
    CTPP: number;
    CSPB: number;
};
//# sourceMappingURL=types.d.ts.map