"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViperChannelType = exports.HomeIndex = exports.STATUS_CLOSED = exports.STATUS_OPEN = exports.STATUS_OFF = exports.STATUS_ON = exports.IDLE = exports.CLOSE = exports.OPEN = exports.OFF = exports.ON = exports.OBJECT_SUBTYPE = exports.OBJECT_TYPE = void 0;
var OBJECT_TYPE;
(function (OBJECT_TYPE) {
    OBJECT_TYPE[OBJECT_TYPE["OTHER"] = 1] = "OTHER";
    OBJECT_TYPE[OBJECT_TYPE["BLIND"] = 2] = "BLIND";
    OBJECT_TYPE[OBJECT_TYPE["LIGHT"] = 3] = "LIGHT";
    OBJECT_TYPE[OBJECT_TYPE["IRRIGATION"] = 4] = "IRRIGATION";
    OBJECT_TYPE[OBJECT_TYPE["THERMOSTAT"] = 9] = "THERMOSTAT";
    OBJECT_TYPE[OBJECT_TYPE["OUTLET"] = 10] = "OUTLET";
    OBJECT_TYPE[OBJECT_TYPE["POWER_SUPPLIER"] = 11] = "POWER_SUPPLIER";
    OBJECT_TYPE[OBJECT_TYPE["ZONE"] = 1001] = "ZONE";
})(OBJECT_TYPE = exports.OBJECT_TYPE || (exports.OBJECT_TYPE = {}));
var OBJECT_SUBTYPE;
(function (OBJECT_SUBTYPE) {
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["GENERIC"] = 0] = "GENERIC";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["DIGITAL_LIGHT"] = 1] = "DIGITAL_LIGHT";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["RGB_LIGHT"] = 2] = "RGB_LIGHT";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["TEMPORIZED_LIGHT"] = 3] = "TEMPORIZED_LIGHT";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["DIMMER_LIGHT"] = 4] = "DIMMER_LIGHT";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["OTHER_DIGIT"] = 5] = "OTHER_DIGIT";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["OTHER_TMP"] = 6] = "OTHER_TMP";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["ELECTRIC_BLIND"] = 7] = "ELECTRIC_BLIND";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["CLIMA_TERM"] = 12] = "CLIMA_TERM";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["GENERIC_ZONE"] = 13] = "GENERIC_ZONE";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["CONSUMPTION"] = 15] = "CONSUMPTION";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["CLIMA_THERMOSTAT_DEHUMIDIFIER"] = 16] = "CLIMA_THERMOSTAT_DEHUMIDIFIER";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["CLIMA_DEHUMIDIFIER"] = 17] = "CLIMA_DEHUMIDIFIER";
    OBJECT_SUBTYPE[OBJECT_SUBTYPE["ENHANCED_ELECTRIC_BLIND"] = 31] = "ENHANCED_ELECTRIC_BLIND";
})(OBJECT_SUBTYPE = exports.OBJECT_SUBTYPE || (exports.OBJECT_SUBTYPE = {}));
exports.ON = 1;
exports.OFF = 0;
exports.OPEN = 1;
exports.CLOSE = 0;
exports.IDLE = 2;
exports.STATUS_ON = '1';
exports.STATUS_OFF = '0';
exports.STATUS_OPEN = '1';
exports.STATUS_CLOSED = '0';
class HomeIndex {
    constructor(home, logger = console) {
        this.othersIndex = new Map();
        this.lightsIndex = new Map();
        this.roomsIndex = new Map();
        this.thermostatsIndex = new Map();
        this.blindsIndex = new Map();
        this.outletsIndex = new Map();
        this.irrigationIndex = new Map();
        this.supplierIndex = new Map();
        this.unknownIndex = new Map();
        this.mainIndex = new Map();
        home.elements.forEach((info) => {
            this.visitElement(info);
            logger.debug(`Added home device with id: ${info.id}, type: ${info.data.type} (sub-type ${info.data.sub_type})`);
        });
    }
    get(id) {
        return this.mainIndex.get(id);
    }
    updateObject(id, data) {
        if (this.mainIndex.has(id)) {
            const deviceData = this.mainIndex.get(id);
            const value = Object.assign(Object.assign({}, deviceData), data);
            this.mainIndex.set(id, Object.freeze(value));
            return value;
        }
        return null;
    }
    visitElement(element) {
        switch (element.data.type) {
            case OBJECT_TYPE.OTHER:
                this.othersIndex.set(element.id, element.data);
                break;
            case OBJECT_TYPE.LIGHT:
                this.lightsIndex.set(element.id, element.data);
                break;
            case OBJECT_TYPE.ZONE:
                this.roomsIndex.set(element.id, element.data);
                break;
            case OBJECT_TYPE.THERMOSTAT:
                this.thermostatsIndex.set(element.id, element.data);
                break;
            case OBJECT_TYPE.BLIND:
                this.blindsIndex.set(element.id, element.data);
                break;
            case OBJECT_TYPE.OUTLET:
                this.outletsIndex.set(element.id, element.data);
                break;
            case OBJECT_TYPE.POWER_SUPPLIER:
                this.supplierIndex.set(element.id, element.data);
                break;
            case OBJECT_TYPE.IRRIGATION:
                this.irrigationIndex.set(element.id, element.data);
                break;
            default:
                this.unknownIndex.set(element.id, element.data);
        }
        if (this.mainIndex.has(element.id)) {
            console.warn(`Overwriting element with key ${element.id} in index!`);
        }
        this.mainIndex.set(element.id, element.data);
        if (element.data.elements) {
            element.data.elements.forEach(value => this.visitElement(value));
        }
    }
}
exports.HomeIndex = HomeIndex;
// ICONA types
// eslint-disable-next-line @typescript-eslint/no-unused-vars
exports.ViperChannelType = {
    SERVER_INFO: 20,
    PUSH: 2,
    UAUT: 2,
    UCFG: 3,
    CTPP: 7,
    CSPB: 8,
};
/*
// eslint-disable-next-line @typescript-eslint/no-unused-vars
const allConfigExample: ConfigurationResponse = {
  message: 'get-configuration',
  'message-type': 'response',
  'message-id': 3,
  'response-code': 200,
  'response-string': 'OK',
  'viper-server': <ViperServer>{
    'local-address': '192.168.0.66',
    'local-tcp-port': 64100,
    'local-udp-port': 64100,
    'remote-address': '',
    'remote-tcp-port': 64100,
    'remote-udp-port': 64100,
  },
  'viper-client': <ViperClient>{description: 'SU0EG'},
  'viper-p2p': {
    mqtt: <MQTTConfig>{
      role: 'a',
      base: 'HSrv/0025291701EC/vip/COMHUB01/sdp',
      server: 'tls://hub-vip3.cloud.comelitgroup.com:443',
      auth: {method: ['CCS_TOKEN', 'CCS_DEVICE']},
    },
    http: <HTTPConfig>{role: 'a', duuid: 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'},
    stun: <STUNConfig>{
      server: ['turn-1-de.cloud.comelitgroup.com:3478', 'turn-1-de.cloud.comelitgroup.com:3478'],
    },
  },
  vip: {
    enabled: true,
    'apt-address': 'COMHUB01',
    'apt-subaddress': 2,
    'logical-subaddress': 2,
    'apt-config': {
      description: '',
      'call-divert-busy-en': false,
      'call-divert-address': '',
      'virtual-key-enabled': false,
    },
    'user-parameters': {
      forced: true,
      'apt-address-book': [],
      'switchboard-address-book': [],
      'camera-address-book': [],
      'rtsp-camera-address-book': [],
      'entrance-address-book': [],
      'actuator-address-book': [],
      'opendoor-address-book': [
        <DoorItem>{name: 'CANCELLO', 'apt-address': '00000100', 'output-index': 2, 'secure-mode': false},
      ],
      'opendoor-actions': [<OpenDoorAction>{action: 'peer', 'apt-address': '', 'output-index': 1}],
    },
  },
  'building-config': <BuildingConfig>{description: 'your building'},
};
*/
//# sourceMappingURL=types.js.map