"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringToBuffer = exports.numberToHex = exports.bytesToHex = exports.sleep = exports.doGet = exports.generateUUID = exports.NULL = void 0;
const crypto_1 = __importDefault(require("crypto"));
const axios_1 = __importDefault(require("axios"));
const lodash_1 = require("lodash");
exports.NULL = Buffer.from([0x00]);
// See https://stackoverflow.com/questions/105034/how-to-create-guid-uuid
function generateUUID(data) {
    const sha1sum = crypto_1.default.createHash('sha1');
    sha1sum.update(data);
    const s = sha1sum.digest('hex');
    let i = -1;
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
        i += 1;
        switch (c) {
            case 'y':
                return ((parseInt('0x' + s[i], 16) & 0x3) | 0x8).toString(16);
            case 'x':
            default:
                return s[i];
        }
    });
}
exports.generateUUID = generateUUID;
axios_1.default.defaults.timeout = 3000;
function doGet(address, path, uid, params = null) {
    return __awaiter(this, void 0, void 0, function* () {
        const resp = yield axios_1.default.get(`${address}${path}`, {
            params,
            headers: {
                Cookie: uid,
                'X-Requested-With': 'XMLHttpRequest',
                Accept: '*/*',
            },
        });
        if (resp.status >= 200 && resp.status < 300) {
            return resp.data;
        }
        throw new Error(`Unable to GET data: ${resp.statusText}`);
    });
}
exports.doGet = doGet;
function sleep(time) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve) => setTimeout(() => resolve(null), time));
    });
}
exports.sleep = sleep;
function bytesToHex(byteArray) {
    const chunks = (0, lodash_1.chunk)(byteArray, 16);
    return chunks.map(c => {
        const hex = c.reduce((output, elem) => output + ('0' + elem.toString(16)).slice(-2) + " ", '');
        const ascii = c.map(c => c <= 128 && c > 31 ? String.fromCharCode(c) : '·').join('');
        return `${(0, lodash_1.padEnd)(hex, 48)}\t${ascii}`;
    }).join('\n');
}
exports.bytesToHex = bytesToHex;
function numberToHex(n) {
    let x = n;
    const byteArray = [];
    while (x !== 0) {
        byteArray.push(x & 0b11111111);
        x = x >> 8;
    }
    return byteArray.reverse().reduce((output, elem) => output + ('0' + elem.toString(16)).slice(-2) + " ", '');
}
exports.numberToHex = numberToHex;
function stringToBuffer(str, nullTerminated = false) {
    const asNumber = [...str].map(c => c.charCodeAt(0));
    return Buffer.from(nullTerminated ? [...asNumber, ...exports.NULL] : [...asNumber]);
}
exports.stringToBuffer = stringToBuffer;
//# sourceMappingURL=utils.js.map