import { ConsoleLike } from './types';
export interface LoginInfo {
    rt_stat: number;
    life: number;
    logged: number;
    vedo_auth: number[];
}
export interface AreaDesc extends LoginInfo {
    present: number[];
    p1_pres: number[];
    p2_pres: number[];
    description: string[];
}
export interface AlarmArea {
    index: number;
    description: string;
    ready: boolean;
    armed: boolean;
    triggered: boolean;
    sabotaged: boolean;
}
export interface AreaStatus extends LoginInfo {
    zone_open: number;
    ready: number[];
    armed: number[];
    alarm: number[];
    alarm_memory: number[];
    sabotage: number[];
    anomaly: number[];
    in_time: number[];
    out_time: number[];
}
export interface ZoneDesc extends LoginInfo {
    present: string;
    in_area: number[];
    description: string[];
}
export interface ZoneStat extends LoginInfo {
    status: string;
}
export interface ZoneStatus {
    index: number;
    description: string;
    open: boolean;
    excluded: boolean;
    isolated: boolean;
    sabotated: boolean;
    alarm: boolean;
    inhibited: boolean;
}
export interface VedoClientConfig {
    login: string;
    login_info: string;
    area_desc: string;
    area_stat: string;
    zone_desc: string;
    zone_stat: string;
    action: string;
    code_param: string;
}
export declare class VedoClient {
    private readonly address;
    private readonly config;
    private logger;
    constructor(address: string, port?: number, config?: Partial<VedoClientConfig>);
    setLogger(logger: ConsoleLike): void;
    private login;
    logout(uid: string): Promise<boolean>;
    loginWithRetry(code: string, maxRetries?: number): Promise<string>;
    isLogged(uid: string): Promise<boolean>;
    areaDesc(uid: string): Promise<AreaDesc>;
    areaStatus(uid: string): Promise<AreaStatus>;
    zoneDesc(uid: string): Promise<ZoneDesc>;
    zoneStatus(uid: string, zones?: ZoneDesc): Promise<ZoneStatus[]>;
    findActiveAreas(uid: string, areas?: AreaDesc): Promise<AlarmArea[]>;
    arm(uid: string, area: number, force?: boolean): Promise<any>;
    disarm(uid: string, area: number): Promise<any>;
    excludeZone(uid: string, zoneIndex: number): Promise<any>;
    includeZone(uid: string, zoneIndex: number): Promise<any>;
    shutdown(uid: string): Promise<boolean>;
}
//# sourceMappingURL=vedo-client.d.ts.map